<?php
Anc_Application::autoload('Anc_Library_Plugin_Adapter');

class Anc_Adapter_Clickbank extends Anc_Library_Plugin_Adapter
{
    protected $_http;

    protected $_categoryId;
    protected $_subcategoryId;
    protected $_affiliateId;
    protected $_password;
    protected $_trackingId;
    protected $_sortOrder;
    protected $_sortReverse = TRUE;
    protected $_searchLanguage;
    protected $_searchIncludeKeywords;
    protected $_searchExcludeKeywords;

    protected $_fetchMerchantRatio = 1.5;

    protected $_baseUrl; // set on login
    protected $_marketPath = '/account/marketplace.htm';
    protected $_api2Path = '/api2/marketplace';
    protected $_loginPath = '/account/login';

    const RESULTS_PER_PAGE = 50;

    public function init()
    {
        parent::init();

        Anc_Application::autoload('Anc_Library_Http_ClientWrapper');
        $this->_http = new Anc_Library_Http_ClientWrapper;

        $this->_http->getClient()->setCookieJar();
    }

    //
    // Category scraping
    //

    public function getCategories()
    {
        try {
            $this->_login();
            $html = $this->_http->get($this->_baseUrl . $this->_marketPath);
        } catch (Anc_Library_Http_Exception $e) {
            $this->_throwError($e->getMessage());
        }

        $result = array();
        if (preg_match_all('|<h1 class="category_accordion_h1" id="(.+?)">.*?<a[^>]*>(.+?)</a>.*?<ul[^>]*>(.+?)</ul>|ism',
            $html, $matches, PREG_SET_ORDER))
        {
            foreach ($matches as $match) {
                $id = $this->_parseCategoryId($match[1]);

                $result[$id] = array(
                    'id' => $id,
                    'label' => $this->_filterCategoryLabel($match[2]),
                    'children' => $this->_parseSubcategories($match[3])
                );
            }
        } else {
            $this->_throwError("Cannot extract category list.");
        }

        return $result;
    }

    protected function _parseSubcategories($list)
    {
        if (preg_match_all('|<li[^>]*>.*?<a id="(.+?)"[^>]*>(.+?)</a>.*?</li>|ism',
            $list, $matches, PREG_SET_ORDER))
        {
            $result = array();
            foreach ($matches as $match) {
                $id = $this->_parseCategoryId($match[1]);
                $result[$id] = array(
                    'id' => $id,
                    'label' => $this->_filterCategoryLabel($match[2]),
                );
            }
        } else {
            $this->_throwError("Parsing error #2.");
        }

        return $result;
    }

    protected function _parseCategoryId($id)
    {
        $parts = explode('_', $id);
        if (count($parts) < 2) {
            $this->_throwError("Parsing error #1.");
        }
        return $parts[1];
    }

    protected function _filterCategoryLabel($label)
    {
        return trim($this->_htmlDecode($label));
    }

    //
    // Mercant data scraping
    //

    public function getMerchants($preview=FALSE)
    {
        $this->_login();

        $exactMaxMerchants = $this->_app->getMaxMerchants();
        $maxMerchants = round($this->_fetchMerchantRatio * $exactMaxMerchants);
        /**
        * locale=EN&includeKeywords="test"+"test1"&excludeKeywords="test2"+"test3"&mainCategoryId=1253&subCategoryIdSource=&resultsPerPage=50&advanced=true
        */
        $params = array(
            'locale' => 'EN',
            'resultsPerPage' => self::RESULTS_PER_PAGE,
            'includeKeywords' =>
                $this->_formatKeywordsParameter($this->_searchIncludeKeywords),
            'excludeKeywords' =>
                $this->_formatKeywordsParameter($this->_searchExcludeKeywords),
            'mainCategoryId' => (string)$this->_categoryId,
            'productLanguages' => $this->_searchLanguage,
            'sortField' => $this->_sortOrder,
            '_sort' => 'on',
            'sortReverse' => $this->_sortReverse? 'true' : 'false',
            'advanced' => 'true',
        );
        if ($this->_subcategoryId) {
            $params['subCategoryId'] = (string)$this->_subcategoryId;
        }

        $this->_publishedCount = 0;
        $result = array();
        $i = 1;

        while (count($result) < $maxMerchants) {
            if (!$this->_processMerchantList($result, $i, $params, $preview, $exactMaxMerchants)) {
                break;
            }

            if ($preview && count($result) - $this->_publishedCount >= $exactMaxMerchants) {
                break;
            }

            $i++;
        }

        return $result;
    }

    protected function _processMerchantList(array &$merchants, $page,
        array $params, $preview, $exactMaxMerchants)
    {
        $this->_http->getClient()->setHeaders(array(
            'Accept' => 'application/json',
            'Page' => $page,
        ));

        try {
            $json = $this->_http->get($this->_baseUrl . $this->_api2Path,
                $params);
        } catch (Anc_Library_Http_Exception $e) {
           $this->_throwError($e->getMessage());
        }

        $json = @json_decode($json);
        if (!$json) {
            $this->_throwError('JSON decoding failed');
        }

        if (!isset($json->result->details)) {
            return FALSE;
        }

        // Use publisher to filter exisitng merchants
        $publisher = $this->_app->getPublisher();

        foreach ($json->result->details as $item) {
            $id = $item->site;

            // Skip existing merchant:
            $published = $publisher->isPublished($id);
            if ($published) {
                $this->_publishedCount++;
                if (!$preview) {
                    continue;
                }
            }

            $merchants[$id] = array(
                'id' => $id,
                'url' => 'http://zzzzz.' . strtolower($id) . '.hop.clickbank.net',
                'affiliateNetwork-title' => $item->title,
            );

            if ($preview) {
                $merchants[$id]['published'] = $published;
                if (count($merchants) - $this->_publishedCount >= $exactMaxMerchants) {
                    return FALSE;
                }
            }
        }

        // Check if we ran out of results
        if (!isset($json->result->totalHitCount)) {
            $this->_throwError('totalHitCount not found');
        }
        if (self::RESULTS_PER_PAGE * $page >= $json->result->totalHitCount) {
            return FALSE;
        }

        usleep(500);

        return TRUE;
    }

    //
    // Helpers
    //

    protected function _htmlDecode($text)
    {
        $result = html_entity_decode($text, ENT_COMPAT, 'UTF-8');
        if (empty($result)) {
            $result = html_entity_decode($text, ENT_QUOTES);
        }

        return $result;
    }

    protected function _formatKeywordsParameter($keywords)
    {
        if (is_null($keywords)) {
            return NULL;
        }
        foreach ($keywords as $i=>$keyword) {
            $keyword = trim($keyword);
            if ('' == $keyword) continue;
            $keywords[$i] = '"' . trim($keyword) . '"';
        }

        return implode(' OR ', $keywords);
    }

    //
    // Common request methods
    //

    public function _login()
    {
        $this->_ensureHasCredentials();

        $id = trim($this->_affiliateId);
        if (!preg_match('~^[a-zA-Z0-9]+$~u', $id, $matches)) {
            $this->_throwError("ClickBank ID '{$id}' contains invalid characters.");
        }
        $id = strtolower($id);

        $this->_baseUrl = "https://{$id}.accounts.clickbank.com";

        $this->_http->post($this->_baseUrl . $this->_loginPath, array(
            'destination' => '/account/mainMenu.htm',
            'nick' => $id,
            'pass' => $this->_password,
            'rememberMe' => 'true',
            'j_username' => $id,
            'j_password' => $this->_password,
        ));

        $cookie = $this->_http->checkCookie('token');
        if (!$cookie) {
            $this->_throwError('Cannot login to ClickBank: Invalid username or password.');
        }
    }

    public function _ensureHasCredentials()
    {
        if ('' == $this->_affiliateId) {
           $this->_throwError('ClickBank ID not specified');
        }
        if ('' == $this->_password) {
           $this->_throwError('ClickBank password not specified');
        }
    }

    //
    // Getters & setters
    //

    public function setCategoryId($value)
    {
        $this->_categoryId = $value;
    }

    public function setSubcategoryId($value)
    {
        $this->_subcategoryId = $value;
    }

    public function setAffiliateId($value)
    {
        $this->_affiliateId = $value;
    }

    public function setPassword($value)
    {
        $this->_password = $value;
    }

    public function setTrackingId($value)
    {
        $this->_trackingId = $value;
    }

    public function setMaxMerchants($value)
    {
        $this->_maxMerchants = $value;
    }

    public function setSortOrder($value)
    {
        $this->_sortOrder = $value;
    }

    public function setSortReverse($value)
    {
        $this->_sortReverse = $value;
    }

    public function setSearchLanguage($value)
    {
        $this->_searchLanguage = $value;
    }

    public function setSearchIncludeKeywords($value)
    {
        $this->_searchIncludeKeywords = $value;
    }

    public function setSearchExcludeKeywords($value)
    {
        $this->_searchExcludeKeywords = $value;
    }
}