<?php
//define('DEBUG', FALSE);

define('ANC_APPLICATION_PATH', dirname(__FILE__));

// Define application environment
if (!defined('ANC_APPLICATION_ENV')) {
    if (getenv('ANC_APPLICATION_ENV')) {
        define('ANC_APPLICATION_ENV', getenv('ANC_APPLICATION_ENV'));
    } else if (getenv('REDIRECT_ANC_APPLICATION_ENV')) {
        define('ANC_APPLICATION_ENV', getenv('REDIRECT_ANC_APPLICATION_ENV'));
    }
    else define('ANC_APPLICATION_ENV', 'production');
}

class Anc_Application
{
    const ADAPTER_NAMESPACE     = 'Anc_Adapter';
    const SCRAPER_NAMESPACE     = 'Anc_Scraper';
    const PUBLISHER_NAMESPACE   = 'Anc_Publisher';
    const FEEDBACK_NAMESPACE    = 'Anc_Feedback';

    /**
    * Application configuration
    *
    * @var CbcZend_Config
    */
    protected $_config;

    // Settings
    protected $_maxMerchants = 100;

    protected $_pingThumbs = FALSE;
    protected $_thumbSize = 'm';

    protected $_merchantList;

    // Never access plugins directly. Use getters to ensure plug-ins are loaded.
    protected $_adapter;
    protected $_scraper;
    protected $_publisher;
    protected $_feedback;

    public function __construct()
    {
        self::autoloadLib('CbcZend_Config_Ini');
        $this->_config = new CbcZend_Config_Ini(realpath(ANC_APPLICATION_PATH . '/config.ini'), ANC_APPLICATION_ENV);
    }

    public function generate()
    {
        error_reporting(E_ERROR|E_PARSE|E_CORE_ERROR|E_COMPILE_ERROR|E_USER_ERROR|E_RECOVERABLE_ERROR);
        ini_set('error_log', dirname(__FILE__) . '/error.log');
        ini_set('log_errors', TRUE);

        set_time_limit(3600);

        ini_set('memory_limit', '512M');
        self::autoload('Anc_Library_PhpError_Handler');
        Anc_Library_PhpError_Handler::enable(E_ERROR|E_WARNING);

        try {
            $this->feedback(NULL, 'progress');
            $this->ping();

            $useMerchantList = is_array($this->_merchantList);

            $adapter = $this->getAdapter();

            if ($useMerchantList) {
                $merchants = $this->_merchantList;
                $this->feedback('Generating posts for selected merchants (' . count($merchants) . ').' . "\n");

                $publishedCount = 0;
            } else {
                // Get merchant list
                $this->feedback('Reading merchant list... ');
                try {
                    $merchants = $adapter->getMerchants();
                } catch (Anc_Library_Plugin_Exception $e) {
                    $this->feedback('Failed: ' . $e->getMessage(), 'error');
                    return;
                }

                $publishedCount = $adapter->getPublishedCount();
            }

            //Release adapter
            $adapter->release();
            $this->_adapter = $adapter = NULL;

            $count = count($merchants);

            $total = $useMerchantList
                ? $count
                : min($count, $this->_maxMerchants);

            if (0 == $count) {
                if ($publishedCount > 0) {
                    $this->feedback("Skipped {$publishedCount} merchants already published.\n");
                    $this->feedback("No new merchants found for the search criteria. You already have posts for all merchants found.", 'error');
                } else {
                    $this->feedback("No merchants found for the search criteria.", 'error');
                }
                return;
            } else if (!$useMerchantList) {
                $this->feedback("fetched {$count} merchants.\n");
                if ($publishedCount > 0) {
                    $this->feedback("Skipped {$publishedCount} merchants already published.\n");
                }
            }

            $scraper = $this->getScraper();
            $publisher = $this->getPublisher();
            $processed = 0;
            $index = 0;

            foreach ($merchants as $id=>$merchant) {
                $this->ping();
                $progress = ($useMerchantList || $total < $this->_maxMerchants)
                    ? $index / $total
                    : $processed / $total;

                $index++;

                $label = isset($merchant['affiliateNetwork-title'])
                    ? "{$merchant['affiliateNetwork-title']} ({$merchant['id']})"
                    : $merchant['id'];

                $this->feedback("Generating post for merchant: {$label} ... ", NULL, $progress);

                try {
                    $scraper->scrape($merchant);
                    $this->feedback();
                    $publisher->publish($merchant);
                } catch (Anc_Library_Plugin_Exception $e) {
                    $this->feedback('failed: ' . $e->getMessage() . "\n");
                    continue;
                }

                $this->feedback(" ok\n");

                $processed++;
                if ($processed >= $this->_maxMerchants) {
                    break;
                }
            }
            $this->feedback("\n");

            $msg = $processed
                ? "{$processed} posts added successfully."
                : 'Unfortunately, found no merchant applicable to add a post.';
            $this->feedback("Finished. {$msg}", 'success', 1.0);
        } catch (Exception $e) {
            Anc_Library_PhpError_Handler::disable();
            $this->feedback("\nUnexpected error: {$e}\n\n");
            $this->feedback("Unexpected error occured. Please contact the support.", 'error');
            @error_log($e, E_ERROR);
            exit;
        }

        Anc_Library_PhpError_Handler::disable();
    }

    public function preview()
    {
        return $this->getAdapter()->getMerchants(TRUE);
    }


    //
    // Getters & setters
    //

    public function getConfig()
    {
        return $this->_config;
    }

    public function getAdapter()
    {
        if (is_null($this->_adapter)) {
            $this->_adapter = $this->_loadPlugin(
                $this->_config->adapter->class, self::ADAPTER_NAMESPACE);
        }
        return $this->_adapter;
    }

    public function getScraper()
    {
        if (is_null($this->_scraper)) {
            $this->_scraper = $this->_loadPlugin(
                $this->_config->scraper->class, self::SCRAPER_NAMESPACE);
        }
        return $this->_scraper;
    }

    public function getPublisher()
    {
        if (is_null($this->_publisher)) {
            $this->_publisher = $this->_loadPlugin(
                $this->_config->publisher->class, self::PUBLISHER_NAMESPACE);
        }
        return $this->_publisher;
    }

    public function getFeedback()
    {
        if (is_null($this->_feedback)) {
            $this->_feedback = $this->_loadPlugin(
                $this->_config->feedback->class, self::FEEDBACK_NAMESPACE);
        }
        return $this->_feedback;
    }

    public function feedback($message='', $status=NULL, $progress=NULL)
    {
        /*
        if (DEBUG) {
            error_log($message);
        }
        */

        $feedback = $this->getFeedback();
        $command = $feedback->exchange($message, $status, $progress);

        if ('terminate' == $command) {
            /*
            if (DEBUG) {
                error_log('TERMINATED');
            }
            */
            $feedback->terminate();
            exit;
            throw Exception('Abort');
        }
    }

    public function setOptions(array $options)
    {
        foreach ($options as $name=>$value) {
            $method = 'set' . ucfirst($name);
            $this->$method($value);
        }
    }

    public function getMaxMerchants()
    {
        return $this->_maxMerchants;
    }

    public function setMaxMerchants($value)
    {
        $this->_maxMerchants = $value;

        return $this;
    }

    public function getPingThumbs()
    {
        return $this->_pingThumbs;
    }

    public function setPingThumbs($value)
    {
        $this->_pingThumbs = $value;

        return $this;
    }

    public function getThumbSize()
    {
        return $this->_thumbSize;
    }

    public function setThumbSize($value)
    {
        $this->_thumbSize = $value;

        return $this;
    }

    public function setMerchantList($value)
    {
        $this->_merchantList = $value;

        return $this;
    }

    //
    // Helpers
    //

    protected function _loadPlugin($class, $namespace)
    {
        if (empty($class)) {
            throw new Exception('Plug-in class not specified.');
        }

        $class = $namespace . '_' . $class;
        self::autoload($class);

        return new $class($this);
    }

    //
    // Static part
    //

    static public $_instance;

    static public function autoload($class)
    {
        $parts = explode('_', $class);
        unset($parts[0]); //remove prefix
        $path = realpath(ANC_APPLICATION_PATH . '/' . implode('/', $parts) . '.php');
        require_once $path;
    }

    static public function autoloadLib($class)
    {
        $parts = explode('_', $class);
        $path = realpath(ANC_APPLICATION_PATH . '/../library/' . implode('/', $parts) . '.php');
        require_once $path;
    }

    static public function app()
    {
        if (!self::$_instance) {
            self::$_instance = new self;
        }

        return self::$_instance;
    }

    /**
    * Workaround for rapidly disconnecting DB servers
    */
    public function ping()
    {
        @mysql_ping();
    }
}